<?php

/**
 * @file
 * Feeds DSpace parser
 */

/**
 * Class definition for DSpace Parser.
 *
 * Parses DSpace REST API feed.
 */
class FeedsDSpaceParser extends FeedsParser {

  /**
   * Implements FeedsParser::parse().
   */
  public function parse(FeedsSource $source, FeedsFetcherResult $fetcher_result) {
    $xml = new SimpleXMLElement($fetcher_result->getRaw());
    $result = new FeedsParserResult();

    $items = $xml->xpath('/items/item');

    foreach ($items as $item) {
      $publication['guid'] = (string) $item->handle;
      $publication['id'] = (string) $item->id;
      $publication['lastmodified'] = (string) $item->lastModified;
      $publication['title'] = (string) $item->name;

      // TODO:  Is this field required?
      // The "submitter" does not appear to be available in the new XML
      // $publication['submitter'] = (string) $item->submitter->netId;

      // Dublin Core Metadata.

      $type = $item->xpath('metadata[key = "dc.type"]/value');
      $publication['type'] = (string) $type[0];

      // This will pull back 2 URI's:  http://arks.princeton.edu/... and http://hdl.handle.net/...
      $identifier = $item->xpath('metadata[key = "dc.identifier.uri"]/value');
      // TODO:  Should this be configurable?!
      // Look for the URI that contains "arks"
      if (substr($identifier[0], 7, 4) == "arks")
      {
        $publication['identifier'] = (string) $identifier[0];
      }
      else
      {
        $publication['identifier'] = (string) $identifier[1];
      }

      $number = $item->xpath('metadata[key = "dc.relation.ispartofseries"]/value');
      if (strpos($number[0], ';') !== false)
      {
        // Get the number at the end of the string
        $publication['number'] = filter_var($number[0], FILTER_SANITIZE_NUMBER_INT);
      }
      else
      {
          $publication['number'] = (string) $number[0];
      }

      $creators = $item->xpath('metadata[key = "dc.creator"]/value');
      foreach ($creators as $creator) {
        $publication['creators'][] = (string) $creator;
      }

      $authors = $item->xpath('metadata[key = "dc.contributor.author"]/value');
      foreach ($authors as $author) {
        $publication['authors'][] = (string) $author;
      }

      $keywords = $item->xpath('/metadata[key = "dc.subject"]/value');
      foreach ($keywords as $keyword) {
        $publication['keywords'][] = (string) $keyword;
      }

      $abstract = $item->xpath('metadata[key = "dc.description.abstract"]/value');
      $publication['abstract'] = (string) $abstract[0];

      $issued = $item->xpath('metadata[key = "dc.date.issued"]/value');
      $publication['issued'] = (string) $issued[0];

      $citation = $item->xpath('metadata[key = "dc.identifier.citation"]/value');
      $publication['citation'] = (string) $citation[0];

      $return_this[] = $publication;

      unset($publication);
    }

    return $result;
  }

  /**
   * Implements FeedsParser::getMappingSources().
   */
  public function getMappingSources() {
    return array(
      'guid' => array(
        'name' => t('GUID'),
        'description' => t('Global Unique Identifier of the feed item.'),
      ),
      'id' => array(
        'name' => t('ID'),
        'description' => t('Identifier of the feed item.'),
      ),
      'lastmodified' => array(
        'name' => t('Last Modified'),
        'description' => t('Last Modified of the feed item.'),
      ),
      'title' => array(
        'name' => t('Title'),
        'description' => t('Title of the feed item.'),
      ),
      'submitter' => array(
        'name' => t('Submitter'),
        'description' => t('Submitter of the feed item.'),
      ),
      'type' => array(
        'name' => t('Type'),
        'description' => t('Publication type of the feed item.'),
      ),
      'identifier' => array(
        'name' => t('Identifier'),
        'description' => t('Identifier of the feed item.'),
      ),
      'number' => array(
        'name' => t('Number'),
        'description' => t('Number of the feed item.'),
      ),
      'creators' => array(
        'name' => t('Creators'),
        'description' => t('Creators of the feed item.'),
      ),
      'authors' => array(
        'name' => t('Authors'),
        'description' => t('Authors of the feed item.'),
      ),
      'keywords' => array(
        'name' => t('Keywords'),
        'description' => t('Keywords of the feed item.'),
      ),
      'abstract' => array(
        'name' => t('Abstract'),
        'description' => t('Abstract of the feed item.'),
      ),
      'issued' => array(
        'name' => t('Issued'),
        'description' => t('Issue Year of the feed item.'),
      ),
      'citation' => array(
        'name' => t('Citation'),
        'description' => t('Bibliographic citation of the feed item.'),
      ),
    )+ parent::getMappingSources();
  }

}
